.TH SPELL 3 "October 18, 1990" "NeXT Computer, Inc."
.SH NAME
.nh
SPbestpath, SPbestmatch, SPmorgan, SPwagfisch - spelling correction functions
.SH SYNOPSIS
.ft L
.B #include <text/spell.h>
.PP
.SH DESCRIPTION
.nh
These functions support spelling correction.
They are part of the common text library, libtext. 
.SH SUMMARY
.nh
char *\fBSPbestpath\fR(char *\fIpath\fR);
.in +.5in
.sp
Attempts to correct misspelled paths;
returns the valid path most closely matching \fIpath\fR,
or zero if no reasonably similar path is found.

To find the best match for \fIpath\fR, fBSPbestpath\fR follows \fIpath\fR
from the top down, using spelling metrics to find the best matches
for invalid components.
Ambiguities are resolved by arbitrarily picking a match.
Two spelling metrics are used: the fast Morgan metric is applied
to match a component in a large directory.  
The slower, but more powerful Wagner/Fischer metric is
applied in smaller directories.
This functions does not tolerate missing components or missing slashes.
.in -.5in
.PP
.nh
char **\fBSPbestmatch\fR(char *\fIs\fR, char *\fItable\fR[], 
.br
.in +1in
int \fItabsize\fR, int (*\fImetric\fR) (char *\fIa\fR, char *\fIb\fR));
.in -1in
.in +.5in
.sp
Finds the best matches for \fIs\fR in a table of strings
and returns them in a static, zero-terminated array.
\fItable\fR is assumed to be zero-terminated, 
but only the first \fItabsize\fR
entries are considered if \fItabsize\fR is given.
\fImetric\fR is a function returning 0 if \fIa\fR and \fIb\fR are identical,
and a larger integer if they are different.
If \fImetric\fR is not supplied, the Wagner/Fischer metric is used.
This default is impractically slow when \fItabsize\fR is larger than about 500.
.in -.5in
.PP
.nh
int
\fBSPmorgan\fR(char *\fIs\fR, char *\fIt\fR);
.in +.5in
.sp
Morgan's spelling metric. Covers 80-85% of all English spelling and typing errors.  This function returns
.in +.5in
.sp
.nf

0	if \fIs\fR and \fIt\fR are identical
1	if one character is incorrect, added or deleted 
	(\fIe.g.\fR, `zbra' vs. `zebra')
2	if two characters are interchanged 
	(\fIe.g.\fR, `zebra' vs. `zerba')
3	otherwise

.fi
.in -.5in
.in -.5in
.PP
.nh
int
\fBSPwagfisch\fR(char *\fIs\fR, char *\fIt\fR);
.in +.5in
.sp
Wagner/Fischer's metric, a generalization of Morgan's
(JACM 21, 1, Jan 1974, pp. 168-178).

Returns the distance between \fIs\fR and \fIt\fR; 
this is the minimum cost of editing \fIs\fR to make \fIt\fR.  
The set of supported editing operations includes:
.in +.5in
.sp
.nf
changing one symbol to another symbol     rit -> rat
deleting a symbol                         bit -> bt
inserting a symbol                        bt  -> bat
.fi
.sp
.in -.5in
The algorithm recognizes two meta characters, \fI*\fR and \fI?\fR .
.in +.5in
.sp
.nf
\fI*\fR	matches any string of zero or more characters
\fI?\fR	matches any single character
.fi
.sp
.in -.5in
For example, \fBSPwagfisch(a*, abcde)\fR is 0.
Note that meta character expansion
affects the commutativity and triangularity of the metric.
This implementation disallows consecutive deletions to improve speed.

Speed may also be improved by making 
the value of the global variable \fBSPmaxdistance\fR small;
this limits the depth of recursion of the algorithm.
Every error costs roughly 3; so setting \fBSPmaxdistance\fR to
\fIn\fR*3 will limit recursion to cases with \fIn\fR errors or fewer.
For \fIn\fR equal to one, the running time is a little longer 
than for \fBSPmorgan\fR, and the results of \fBSPbestmatch\fR are similar.
.SH "SEE ALSO"
text(3)
